@@ -4,40 +4,47 @@ use 5.006;
 use strict;
 use warnings;
 
-use base qw ( Exporter );
-our @EXPORT_OK = qw( detect get_categories add_clues_file );
-
 use lib::abs;
 use JSON qw();
 use Regexp::Parser;
 
-my %_categories;
-my @_clues_file_list = ( lib::abs::path( './apps.json' )  );
-
-# List of multi per web page application categories
-my %MULTIPLE_APP_CATS = map { $_ => 1 } qw( 
-    widgets analytics javascript-frameworks video-players
-    font-scripts miscellaneous advertising-networks payment-processors
+# List of multi per-page application categories names
+my %MULTIPLE_APP_CATS = map { $_ => 1 } (
+    'Widgets',
+    'Analytics',
+    'JavaScript frameworks',
+    'JavaScript libraries',
+    'UI frameworks',
+    'Video players',
+    'Font scripts',
+    'Miscellaneous',
+    'Advertising',
+    'Payment processors',
+    'JavaScript graphics',
+    'Marketing automation',
+    'Web server extensions',
+    'WordPress plugins',
 );
 
 =head1 NAME
 
-WWW::Wappalyzer - Perl port of Wappalyzer (http://wappalyzer.com)
+WWW::Wappalyzer - Perl port of Wappalyzer (https://wappalyzer.com)
 
 =head1 DESCRIPTION
 
 Uncovers the technologies used on websites: detects content management systems, web shops,
 web servers, JavaScript frameworks, analytics tools and many more.
 
-Lacks 'version', 'implies', 'excludes' support of original Wappalyzer in favour of speed.
+Supports only `scriptSrc`, `scripts`, `html`, `meta`, `headers`, 'cookies' and `url` patterns of
+Wappalyzer specification. Lacks 'version', 'implies', 'excludes' support in favour of speed.
 
-Clues:      L<https://github.com/AliasIO/Wappalyzer/blob/master/src/apps.json>
-
-More info on Wappalyzer:  L<https://github.com/AliasIO/Wappalyzer>
+Categories: L<https://github.com/wappalyzer/wappalyzer/blob/master/src/categories.json>
+Technologies: L<https://github.com/wappalyzer/wappalyzer/tree/master/src/technologies>
+More info on Wappalyzer:  L<https://github.com/wappalyzer/wappalyzer>
 
 =cut
 
-our $VERSION = '0.21';
+our $VERSION = '2.00';
 
 =head1 SYNOPSIS
 
@@ -46,17 +53,20 @@ our $VERSION = '0.21';
     use List::Util 'pairmap';
 
     my $response = LWP::UserAgent->new->get( 'http://www.drupal.org' );
-    my %detected = WWW::Wappalyzer::detect(
+    my %detected = WWW::Wappalyzer->new->detect(
         html    => $response->decoded_content,
         headers => { pairmap { $a => [ $response->headers->header($a) ] } $response->headers->flatten },
     );
 
     # %detected = (
-    #     'web-servers'       => [ 'Apache' ],
-    #     'cms'               => [ 'Drupal' ],
-    #     'cache-tools'       => [ 'Varnish' ],
-    #     'analytics'         => [ 'Google Analytics' ],
-    #     'operating-systems' => [ 'CentOS' ]
+    #   'Font scripts'    => [ 'Google Font API' ],
+    #   'Caching'         => [ 'Varnish' ],
+    #   'CDN'             => [ 'Fastly' ],
+    #   'CMS'             => [ 'Drupal' ],
+    #   'Video players'   => [ 'YouTube' ],
+    #   'Tag managers'    => [ 'Google Tag Manager' ],
+    #   'Reverse proxies' => [ 'Nginx' ],
+    #   'Web servers'     => [ 'Nginx' ],
     # );
 
 =head1 EXPORT
@@ -65,35 +75,74 @@ None by default.
 
 =head1 SUBROUTINES/METHODS
 
+=head2 new
+
+    my $wappalyzer = WWW::Wappalyzer->new( %params )
+
+Constructor.
+
+Available parameters:
+
+    categories   - optional additional categories array ref to files list (refer 'add_categories_files' below)
+    technologies - optional additional technologies array ref to files list (refer 'add_technologies_files' below)
+
+Returns the instance of WWW::Wappalyzer class.
+
+=cut
+
+sub new {
+    my ( $class, %params ) = @_;
+
+    my $self = bless {
+        _categories      => {},
+        _cats_file_list  => [],
+        _techs_file_list => [],
+    }, $class;
+
+    $self->add_categories_files( lib::abs::path( './wappalyzer_src/categories.json' ) );
+    $self->add_technologies_files( glob lib::abs::path( './wappalyzer_src/technologies' ) . '/*.json'  );
+
+    if ( ref $params{categories} eq 'ARRAY' ) {
+        $self->add_categories_files( @{ $params{categories} } );
+    }
+
+    if ( ref $params{technologies} eq 'ARRAY' ) {
+        $self->add_technologies_files( @{ $params{technologies} } );
+    }
+
+    return $self;
+}
+
 =head2 detect
 
-    my %detected = detect( %params )
+    my %detected = $wappalyzer->detect( %params )
 
-Tries to detect CMS, framework, etc for given html code, http headers, url.
+Tries to detect CMS, framework, etc for given html code, http headers, URL.
 
 Available parameters:
 
     html    - HTML code of web page.
 
-    headers - Hash ref to http headers list. The value may be a plain string or a array ref 
+    headers - Hash ref to http headers list. The value may be a plain string or an array ref
               of strings for a multi-valued field.
+              Cookies should be passed in 'Set-Cookie' header.
 
     url     - URL of web page.
 
-    cats    - Array ref to a list of trying categories, defaults to all categories;
-              Less cats => less cpu usage.
+    cats    - Array ref to a list of trying categories names, defaults to all.
+              Less categories - less CPU usage.
 
 Returns the hash of detected applications by category:
 
     (
-        cms  => [ 'Joomla' ],
-        'javascript-frameworks' => [ 'jQuery', 'jQuery UI' ],
+        CMS  => [ 'Joomla' ],
+        'Javascript frameworks' => [ 'jQuery', 'jQuery UI' ],
     )
 
 =cut
 
 sub detect {
-    my %params = @_;
+    my ( $self, %params ) = @_;
 
     return () unless $params{html} || $params{headers} || $params{url};
     
@@ -124,16 +173,25 @@ sub detect {
         }
     }
 
-    # Lazy load and process clues from JSON file
-    _load_categories()  unless scalar keys %_categories;
+    # Lazy load and process techs from JSON file
+    $self->_load_categories_and_techs  unless scalar keys %{ $self->{_categories} };
 
     my @cats = $params{cats} && ( ref( $params{cats} ) || '' ) eq 'ARRAY'
-        ? @{ $params{cats} } : get_categories();
+        ? @{ $params{cats} } : $self->get_categories_names;
+
+
+    my %cookies;
+    if ( my $cookies_header = $headers_ref->{'set-cookie'} ) {
+        for my $cookie_str ( @$cookies_header ) {
+            next  unless $cookie_str =~ /^(?<name>.+?)=(?<value>.*?);\s/;
+            $cookies{ lc $+{name} } = $+{value};
+        }
+    }
 
     my %detected;
     my %tried_multi_cat_apps;
     for my $cat ( @cats ) {
-        my $apps_ref = $_categories{ $cat } or die "Unknown category $cat";
+        my $apps_ref = $self->{_categories}{ $cat } or die "Unknown category name $cat";
 
         APP:
         for my $app_ref ( @$apps_ref ) {
@@ -169,11 +227,28 @@ sub detect {
                     }
                 }
 
+                if ( !$detected && exists $app_ref->{cookies_rules} && scalar keys %cookies ) {
+                    my %cookies_rules = %{ $app_ref->{cookies_rules} };
+
+                    COOKIE_RULE:
+                    while ( my ( $cookie, $rule ) = each %cookies_rules ) {
+                        my $cookie_val = $cookies{ $cookie } or next;
+
+                        if ( $cookie_val =~ /$rule->{re}/ ) {
+                            $confidence += $rule->{confidence};
+                            if ( $confidence >= 100 ) {
+                                $detected = 1;
+                                last COOKIE_RULE;
+                            }
+                        }
+                    }
+                }
+
                 unless ( $detected ) {
                     # try from most to least relevant method
                     RULES:
                     for my $rule_type ( qw( html url ) ) {
-                        my $rule_name = $rule_type. '_rules';
+                        my $rule_name = $rule_type . '_rules';
                         if ( defined $params{ $rule_type } && exists $app_ref->{ $rule_name } ) {
                             for my $rule ( @{ $app_ref->{ $rule_name } } ) {
                                 if ( $params{ $rule_type } =~ m/$rule->{re}/ ) {
@@ -206,83 +281,99 @@ sub detect {
     return %detected;
 }
 
-=head2 get_categories
+=head2 get_categories_names
 
-    my @cats = get_categories()
+    my @cats = $wappalyzer->get_categories_names()
 
-Returns the array of all application categories.
+Returns the array of all application categories names.
 
 =cut
 
-sub get_categories {
-    # Lazy load and process clues from JSON files
-    _load_categories() unless scalar keys %_categories;
+sub get_categories_names {
+    my ( $self ) = @_;
 
-    return keys %_categories;
+    # Lazy load and process categories from JSON files
+    $self->_load_categories_and_techs()  unless scalar keys %{ $self->{_categories} };
+
+    return keys %{ $self->{_categories} };
 }
 
-# Loads and processes clues from JSON files
-sub _load_categories {
+# Loads and processes categories and techs from JSON files
+sub _load_categories_and_techs {
+    my ( $self ) = @_;
 
-    for my $clue_file ( @_clues_file_list ) {
-        open my $fh, '<', $clue_file
-            or die "Can not read clues file $clue_file.";
+    my $cats_ref = {};
 
-        local $/ = undef;
-        my $json = <$fh>;
-        close $fh;
+    for my $cats_file ( @{ $self->{_cats_file_list} } ) {
+       $cats_ref = { %$cats_ref, %{ _load_json( $cats_file ) } };
+    }
 
-        # Replace html entities with oridinary symbols
-        $json =~ s{&gt;}{>}xig;
-        $json =~ s{&lt;}{<}xig;
-
-        my $cfg_ref = eval { JSON::decode_json( $json ) };
-
-        die "Can't parse clue file $clue_file: $@" if $@;
-
-        my $cats_ref = $cfg_ref->{categories}
-            or die "Broken clues file $clue_file. Can not find categories.";
-
-        my $apps_ref = $cfg_ref->{apps}
-            or die "Broken clues file $clue_file. Can not find applications.";
+    for my $techs_file ( @{ $self->{_techs_file_list} } ) {
+        my $apps_ref = _load_json( $techs_file );
 
         # Process apps
         while ( my ( $app, $app_ref ) = each %$apps_ref ) {
 
-            my $new_app_ref = _process_app_clues( $app, $app_ref ) or next;
+            my $new_app_ref = _process_app_techs( $app, $app_ref ) or next;
 
             my @cats = @{ $app_ref->{cats} } or next;
 
             $new_app_ref->{multi_cat} = 1 if @cats > 1;
 
             for my $cat_id ( @cats ) {
-                my $cat = $cats_ref->{ $cat_id } or next;
+                my $cat = $cats_ref->{ $cat_id } or die "Bad categorie id $cat_id in app $app";
 
-                push @{ $_categories{ $cat } }, $new_app_ref;
+                push @{ $self->{_categories}{ $cat->{name} } }, $new_app_ref;
             }
         }
     }
 }
 
-# Process clues of given app
-sub _process_app_clues {
+# Loads JSON file
+sub _load_json {
+    my ( $file ) = @_;
+
+    open my $fh, '<', $file or die "Can not read file $file.";
+
+    local $/ = undef;
+    my $json = <$fh>;
+    close $fh;
+
+    # Replace html entities with oridinary symbols
+    $json =~ s{&gt;}{>}xig;
+    $json =~ s{&lt;}{<}xig;
+
+    my $res = eval { JSON::decode_json( $json ) };
+
+    die "Can't parse JSON file $file: $@" if $@;
+
+    die "$file has invalid format"  unless ref $res eq 'HASH';
+
+    return $res;
+}
+
+# Process techs of given app
+sub _process_app_techs {
     my ( $app, $app_ref ) = @_;
 
     my $new_app_ref = { name => $app };
 
-    my @fields = grep { exists $app_ref->{$_} } qw( script html meta headers url );
+    my @fields = grep { exists $app_ref->{ $_ } }
+        qw( scriptSrc scripts html meta headers cookies url );
+
     my @html_rules;
+
     # Precompile regexps
     for my $field ( @fields ) {
         my $rule_ref = $app_ref->{ $field };
-        my @rules_list =   !ref $rule_ref ? _parse_rule( $rule_ref )
+        my @rules_list = !ref $rule_ref ? _parse_rule( $rule_ref )
             : ref $rule_ref eq 'ARRAY' ? ( map { _parse_rule( $_ ) } @$rule_ref )
             : () ;
 
-        if ( $field eq 'html' ) {
+        if ( $field eq 'html' || $field eq 'scripts' ) {
             push @html_rules, map { $_->{re} = qr/(?-x:$_->{re})/; $_ } @rules_list;
         }
-        elsif ( $field eq 'script' ) {
+        elsif ( $field eq 'scriptSrc' ) {
             push @html_rules,
                 map {
                     $_->{re} = qr/
@@ -299,7 +390,7 @@ sub _process_app_clues {
             for my $key ( keys %$rule_ref ) {
                 my $lc_key = lc $key;
                 my $name_re = qr/ name \s* = \s* ["']? $lc_key ["']? /x;
-                my $rule = _parse_rule( $rule_ref->{$key} );
+                my $rule = _parse_rule( $rule_ref->{ $key } );
                 $rule->{re} = qr/$rule->{re}/;
                 my $content_re = qr/ content \s* = \s* ["'] [^"']* (?-x:$rule->{re}) [^"']* ["'] /x;
 
@@ -314,11 +405,11 @@ sub _process_app_clues {
                 push @html_rules, $rule;
             }
         }
-        elsif ( $field eq 'headers' ) {
+        elsif ( $field eq 'headers' || $field eq 'cookies' ) {
             for my $key ( keys %$rule_ref ) {
-                my $rule = _parse_rule( $rule_ref->{$key} );
+                my $rule = _parse_rule( $rule_ref->{ $key } );
                 $rule->{re} = qr/$rule->{re}/;
-                $new_app_ref->{headers_rules}{ lc $key } = $rule;
+                $new_app_ref->{ $field . '_rules' }{ lc $key } = $rule;
             }
         }
     }
@@ -334,11 +425,11 @@ sub _process_app_clues {
 sub _parse_rule {
     my ( $rule ) = @_;
     
-    my ( $re, @params ) = split /\\;/, $rule;
+    my ( $re, @tags ) = split /\\;/, $rule;
     
     my $confidence;
-    for my $param ( @params ) {
-        if ( ( $confidence ) = $param =~ /^\s*confidence\s*:\s*(\d+)\s*$/ ) {
+    for my $tag ( @tags ) {
+        if ( ( $confidence ) = $tag =~ /^\s*confidence\s*:\s*(\d+)\s*$/ ) {
             # supports only confidence for now
             last;
         }
@@ -405,24 +496,59 @@ sub _optimize_rules {
     return $rules;
 }
 
-=head2 add_clues_file
+=head2 add_categories_files
 
-    add_clues_file( $filepath )
+    $wappalyzer->add_categories_files( @filepaths )
 
-Puts additional clues file to a list of processed clues files.
-See apps.json as format sample.
+Puts additional categories files to a list of processed categories files.
+See lib/WWW/wappalyzer_src/categories.json as format sample.
 
 =cut
 
-sub add_clues_file {
-    my ( $filepath ) = @_;
+sub add_categories_files {
+    my ( $self, @filepaths ) = @_;
 
-    push @_clues_file_list, $filepath;
+    push @{ $self->{_cats_file_list} }, @filepaths;
 
     # just clear out categories to lazy load later
-    %_categories = ();
+    $self->{_categories} = {};
 }
 
+=head2 add_technologies_files
+
+    $wappalyzer->add_technologies_files( @filepaths )
+
+Puts additional techs files to a list of processed techs files.
+See lib/WWW/wappalyzer_src/technologies/a.json as format sample.
+
+=cut
+
+sub add_technologies_files {
+    my ( $self, @filepaths ) = @_;
+
+    push @{ $self->{_techs_file_list} }, @filepaths;
+
+    # just clear out categories to lazy load later
+    $self->{_categories} = {};
+}
+
+=head2 reload_files
+
+    $wappalyzer->reload_files()
+
+Ask to reload data from additional categories and technologies files
+those may be changed in runtime.
+
+=cut
+
+sub reload_files {
+    my ( $self ) = @_;
+
+    # just clear out categories to lazy load later
+    $self->{_categories} = {};
+}
+
+
 =head1 AUTHOR
 
 Alexander Nalobin, C<< <alexander at nalobin.ru> >>
